#!/usr/bin/env python

from os.path import abspath, commonprefix, exists, join, split
from os import makedirs
from vCalendar import iterwrite

STORE_DIR = "/var/lib/imip-agent/store"

def check_dir(base, dir):
    return commonprefix([base, abspath(dir)]) == base

def to_stream(out, fragment, objtype, encoding="utf-8"):
    iterwrite(out, encoding=encoding).write(objtype, {}, fragment)

class FileStore:

    "A file store of tabular data."

    def __init__(self):
        self.store_dir = STORE_DIR
        if not exists(self.store_dir):
            makedirs(self.store_dir)

    def get_file_object(self, base, *parts):
        pathname = join(base, *parts)
        return check_dir(base, pathname) and pathname or None

    def get_event(self, user, uid):

        "Get the event for the given 'user' with the given 'uid'."

        filename = self.get_file_object(self.store_dir, user, uid)
        if not filename:
            return None

        return exists(filename) and open(filename) or None

    def set_event(self, user, uid, details):

        "Set an event for 'user' having the given 'uid' and 'details'."

        dir = self.get_file_object(self.store_dir, user)
        if not dir:
            return False

        filename = self.get_file_object(dir, uid)
        if not filename:
            return False

        if not exists(dir):
            makedirs(dir)

        f = open(filename, "w")
        try:
            to_stream(f, details, "VEVENT")
        finally:
            f.close()

        return True

    def get_freebusy(self, user):

        "Get free/busy details for the given 'user'."

        filename = self.get_file_object(self.store_dir, user, "freebusy")
        if not filename or not exists(filename):
            return None

        f = open(filename)
        try:
            l = []
            for line in f.readlines():
                l.append(line.strip().split("\t"))
            return l
        finally:
            f.close()

    def set_freebusy(self, user, freebusy):

        "For the given 'user', set 'freebusy' details."

        filename = self.get_file_object(self.store_dir, user, "freebusy")
        if not filename:
            return False

        dir = split(filename)[0]

        if not exists(dir):
            makedirs(dir)

        f = open(filename, "w")
        try:
            for item in freebusy:
                f.write("\t".join(item) + "\n")
        finally:
            f.close()

        return True

# vim: tabstop=4 expandtab shiftwidth=4
